<?php
/*
 * Simple Portal Extra Shoutbox Options v1.72x
 * Developed for SMF forums c/o Chen Zhen @ http://web-develop.ca
 * Copyright 2013 - 2018 noreply@web-develop.ca
 * This software package is distributed under the terms of the GNU General Public License
 * https://www.gnu.org/licenses/gpl.txt
 */

if (!defined('SMF'))
	die('Hacking attempt...');

/*	This file has the job of taking care of a popup editor for
	shoutbox text.  It does this with these functions:

	void sportal_show_popup()
		- shows a popup for editing a specific shout id.
		- uses the portal_popup sub template, no layers.
		- accessed via ?action=portalpopup;shout=??.

	void sportal_edit_shout()
		- loads the specific shout data
		- determines if the user has permission to edit it.

	void sportal_shoutbox_content()
		- configures a return array with smileys and bbcodes
*/

// Show a simple popup editor for a specific opted shout id ...
function sportal_show_popup()
{
	global $txt, $helptxt, $context, $scripturl, $sourcedir, $smcFunc, $settings, $modSettings, $user_info;
	loadLanguage('SPortal');
	require_once($sourcedir . '/Subs-Portal.php');
	require_once($sourcedir . '/Subs-Post.php');
	require_once($sourcedir . '/PortalShoutbox.php');

	if (empty($_REQUEST['shout']) && empty($_POST['shout']))
		fatal_lang_error('error_sp_cannot_shoutbox_edit', false);

	$shout_id = !empty($_REQUEST['shout']) ? (int)$_REQUEST['shout'] : (int)$_POST['shout'];
	$shout_data = sportal_edit_shout($shout_id);
	$shoutbox = sportal_shoutbox_content();

	if (empty($shout_data['can_edit']))
		fatal_lang_error('error_sp_cannot_shoutbox_edit', false);


	// Set the necessary context.
	$context['page_title'] = $context['forum_name'] . ' - ' . $txt['sp_shoutbox_edit_shout'];
	$context['portal_text_id'] = $shout_id;
	$context['portal_current_text'] = $shout_data['body'];
	$context['portal_current_charlimit'] = $shout_data['charlimit'];
	$context['portal_popup_close'] = '
		<script type="text/javascript"><!-- // --><![CDATA[
			function sp_collapse_object(objt)
			{
				var smileys = document.getElementById("sp_object_sb_smiley");
				var bbcs = document.getElementById("sp_object_sb_style");
				var text = document.getElementById("sp_object_sb_text");
				if (objt == "sb_smiley")
					smileys.style.display == "block" ? smileys.style.display = "none" : smileys.style.display = "block";
				else if (objt == "sb_style")
					bbcs.style.display == "block" ? bbcs.style.display = "none" : bbcs.style.display = "block";

				bbcs.style.display == "block" || smileys.style.display == "block" ? text.style.display = "none" : text.style.display = "block";
				return true;
			}
		// ]]></script>';

	$context['portal_current_view'] = '
			<span style="margin-left: 5px;margin-top: 10px;">
				<a href="#smiley" onclick="sp_collapse_object(\'sb_smiley\', false); return false;">' . sp_embed_image('smiley') . '</a> <a href="#style" onclick="sp_collapse_object(\'sb_style\', false); return false;">' . sp_embed_image('style') . '</a>
			</span>
			<div style="height: 40px;">
				<div id="sp_object_sb_text" style="display: block;">
					<div style="display: block;margin-left: 5px;">' . $txt['sp_shoutbox_edit_name'] . '&nbsp;' . $shout_data['name'] . '</div>
					<div style="display: block;margin-left: 5px;">' . $txt['sp_shoutbox_edit_time'] . '&nbsp;' . $shout_data['log_time'] . '</div>
				</div>
				<div id="sp_object_sb_smiley" style="display: none;">';

		foreach ($shoutbox['smileys']['normal'] as $smiley)
			$context['portal_current_view'] .= '
					<a href="javascript:void(0);" onclick="replaceText(\' ' . $smiley['code'] . '\', document.getElementById(\'edit\')); return false;"><img src="' . $settings['smileys_url'] . '/' . $smiley['filename'] . '" alt="' . $smiley['description'] . '" title="' . $smiley['description'] . '" /></a>';

	$context['portal_current_view'] .= '
				</div>
				<div id="sp_object_sb_style" style="display: none;">';

		foreach ($shoutbox['bbc'] as $image => $tag)
		{
			if (!in_array(trim($tag['code']), $shout_data['allowed_bbc']))
				continue;

			if (!isset($tag['after']))
				$context['portal_current_view'] .= '
					<a href="javascript:void(0);" onclick="replaceText(\'' . $tag['before'] . '\', document.getElementById(\'edit\')); return false;">';
			else
				$context['portal_current_view'] .= '
					<a href="javascript:void(0);" onclick="surroundText(\'' . $tag['before'] . '\', \'' . $tag['after'] . '\', document.getElementById(\'edit\')); return false;">';

			$context['portal_current_view'] .= '<img onmouseover="style_highlight(this, true);" onmouseout="if (window.style_highlight) style_highlight(this, false);" src="' . $settings['images_url'] . '/bbc/' . $image . '.gif" align="bottom" width="23" height="22" alt="' . $tag['description'] . '" title="' . $tag['description'] . '" style="background-image: url(' . $settings['images_url'] . '/bbc/bbc_bg.gif); margin: 1px 2px 1px 1px;" /></a>';
		}

	$context['portal_current_view'] .= '
				</div>
			</div>';

	$container_style = array(
		'position: relative',
		'vertical-align: middle',
		'overflow-x: hidden',
		'overflow-y: hidden',
		'margin: 0',
		'padding: 0',
		'-webkit-border-top-left-radius: 10px',
		'-webkit-border-top-right-radius: 10px',
		'-moz-border-radius-topleft: 10px',
		'-moz-border-radius-topright: 10px',
		'border-top-left-radius: 10px',
		'border-top-right-radius: 10px',
		'-webkit-border-bottom-left-radius: 10px',
		'-webkit-border-bottom-right-radius: 10px',
		'-moz-border-radius-bottomleft: 10px',
		'-moz-border-radius-bottomright: 10px',
		'border-bottom-left-radius: 10px',
		'border-bottom-right-radius: 10px',
		'margin-bottom: 0px',
	);
	$context['portal_current_style'] = implode('; ', $container_style);

	if (isset($_POST['submit']))
	{
		if (isset($_POST['edit']))
		{
			checkSession('post');

			// apply opted filters
			if (!allowedTo('spshout_link_perm'))
			{
				$search = array('/\[url\](.*?)\[\/url\]/is'  . ($context['utf8'] ? 'u' : ''), '/\[url\=(.*?)\](.*?)\[\/url\]/is'  . ($context['utf8'] ? 'u' : ''), '/\<a(.*)\>(.*)\<\/a\>/iU'  . ($context['utf8'] ? 'u' : ''), "/(?i)\b((?:https?:\/\/|www\d{0,3}[.]|[a-z0-9.\-]+[.][a-z]{2,4}\/)(?:[^\s()<>]+|\(([^\s()<>]+|(\([^\s()<>]+\)))*\))+(?:\(([^\s()<>]+|(\([^\s()<>]+\)))*\)|[^\s`!()\[\]{};:'\".,<>?]))/"  . ($context['utf8'] ? 'u' : ''));
				$newShout = preg_replace($search, "$2", $_POST['edit']);
				$_POST['edit'] = $newShout;
			}
			$exempt = !empty($shout_data['admin_exempt']) && $user_info['is_admin'] ? true : false;
			$encoding = !empty($context['character_set']) ? $context['character_set'] : 'UTF-8';
			if (allowedTo('spshout_regex_perm') && !empty($shout_data['lowercase']) && !$exempt)
			{
				$_POST['edit'] = preg_replace_callback('~((?<=\s|^)[A-Z]{2,})|(\b[^IO]\b)|(\b[a-z]\w+)|(\b([A-Z]\K([A-Za-z])+)\b)~' . ($context['utf8'] ? 'u' : ''), function($m) use($encoding) {return mb_strtolower($m[0], $encoding);}, $_POST['edit']);
				$_POST['edit'] = preg_replace_callback('~\s:d|\s:p|\s;d|\s:O|\s:-x\b~' . ($context['utf8'] ? 'u' : ''), 'sportal_shoutbox_smiley', $_POST['edit']);
			}
			if (allowedTo('spshout_regex_perm') && !empty($shout_data['regex']) && !empty($shout_data['regex_callback']) && !$exempt)
			{
				$modSettings['sportal_callback'] =  'return ' . trim($shout_data['regex_callback'], 'return ; ') . ';';
				$_POST['edit'] = preg_replace_callback('~' . $shout_data['regex'] . '~' . ($context['utf8'] ? 'u' : ''), 'sportal_shoutbox_callback', $_POST['edit']);
			}

			$bodyShout = $smcFunc['htmlspecialchars'](trim($_POST['edit']));
			preparsecode($bodyShout);
			$bodyShout = preg_replace('~(<br( />)|<br>)~i', ' ', $bodyShout);

			if ($bodyShout !== $shout_data['body'] && strlen(($context['utf8'] ? utf8_decode($bodyShout) : $bodyShout)) > 0)
			{
				// update shout
				$smcFunc['db_query']('', '
					UPDATE {db_prefix}sp_shouts
					SET body = {string:body}
					WHERE id_shout = {int:shout_id}',
					array(
						'body' => $bodyShout . '<div id="edit_shout_' . $shout_id . '" style="display: inline;"><br /><span class="smalltext shoutbox_time" style="font-style: italic;">' . $shout_data['logged_edit'] . '</span></div>',
						'shout_id' => (int)$shout_id,
					)
				);

				sportal_update_shoutbox($shout_data['id_shoutbox']);
			}
		}

		$context['portal_popup_close'] .= '
		<script type="text/javascript"><!-- // --><![CDATA[
			document.getElementById("portal_popup_close").click();
		// ]]></script>';
	}

	// Don't show any template layers, just the portal_popup sub template.
	loadTemplate('PortalEdit');
	$context['template_layers'] = array();
	$context['sub_template'] = 'portal_popup';
}

function sportal_edit_shout($shout_id)
{
	global $smcFunc, $user_info, $txt;

	$can_moderate = allowedTo('sp_admin') || allowedTo('sp_manage_shoutbox');
	list($idMemberShout, $nameMemberShout, $bodyShout, $logTime, $charLimit, $allowed, $shoutId, $adminExempt, $lowercase, $regex, $regexCallback) = array(0, '', '', '', 0, '', $shout_id, 1, 0, '', '');

	if ($user_info['is_guest'])
	{
		$guest = $user_info['name'];
		$_SESSION['portal_guest_shout'] = !empty($_SESSION['portal_guest_shout']) ? $_SESSION['portal_guest_shout'] : rand(pow(10, 3), pow(10, 4)-1);
		$user_info['id'] = -$_SESSION['portal_guest_shout'];
		$user_info['name'] .= $_SESSION['portal_guest_shout'];
	}

	$request = $smcFunc['db_query']('','
		SELECT sh.id_member, sh.member_name, sh.body, sh.id_shoutbox, sh.log_time,
		bx.allowed_bbc, bx.charlimit, bx.admin_exempt, bx.lowercase, bx.regex, bx.regex_callback
		FROM {db_prefix}sp_shouts AS sh
		LEFT JOIN {db_prefix}sp_shoutboxes AS bx ON (sh.id_shoutbox = bx.id_shoutbox)
		WHERE id_shout = {int:shout_id}',
		array(
			'shout_id' => $shoutId,
		)
	);
	while ($row = $smcFunc['db_fetch_assoc']($request))
		list($idShoutbox, $idMemberShout, $nameMemberShout, $bodyShout, $logTime, $charLimit, $allowed, $adminExempt, $lowercase, $regex, $regexCallback) = array((int)$row['id_shoutbox'], (int)$row['id_member'], $row['member_name'], $row['body'], $row['log_time'], (int)$row['charlimit'], $row['allowed_bbc'], $row['admin_exempt'], $row['lowercase'], $row['regex'], $row['regex_callback']);

	$smcFunc['db_free_result']($request);

	$guestId = $user_info['is_guest'] && !empty($nameMemberShout) ? substr($nameMemberShout, -4) : 0;

	$logged_edit = str_replace('%$#@', timeformat(time()), $txt['sp_shoutbox_who_edit_shout']) . (!empty($user_info['name']) ? $user_info['name'] : $user_info['username']);
	$logTime = timeformat($logTime);
	$bodyShout = preg_replace('~<div id="edit_shout_' . $shout_id . '" style="display: inline;">(.*?)</div>(?!\s*<div id="edit_shout_' . $shout_id . '" style="display: inline;">)~', '', $bodyShout);
	$allowed_bbc = !empty($allowed) ? explode(',', $allowed) : array();

	if ($user_info['id'] == $idMemberShout || !empty($can_moderate))
		$can_editx = allowedTo('spshout_edit_perm') ? true : false;
	elseif ($user_info['id'] == -(int)$guestId)
		$can_editx = allowedTo('spshout_edit_perm') ? true : false;
	else
		$can_editx = false;

	if ($user_info['is_guest'])
	{
		$user_info['id'] = 0;
		$user_info['name'] = $guest;
	}

	return array(
		'id_shoutbox' => $idShoutbox,
		'id_shout' => $shout_id,
		'id_member' => $idMemberShout,
		'name' => $nameMemberShout,
		'body' => $bodyShout,
		'log_time' => $logTime,
		'charlimit' => $charLimit,
		'allowed_bbc' => $allowed_bbc,
		'can_edit' => $can_editx,
		'logged_edit' => $logged_edit,
		'admin_exempt' => $adminExempt,
		'lowercase' => $lowercase,
		'regex' => $regex,
		'regex_callback' => $regexCallback,
	);
}

function sportal_shoutbox_content($shoutbox = array())
{
	global $settings, $modSettings, $user_info, $txt, $smcFunc;
	loadLanguage('Post');

	$settings['smileys_url'] = $modSettings['smileys_url'] . '/' . $user_info['smiley_set'];
	$shoutbox['smileys'] = array('normal' => array(), 'popup' => array());
	if (empty($modSettings['smiley_enable']))
		$shoutbox['smileys']['normal'] = array(
			array('code' => ':)', 'filename' => 'smiley.gif', 'description' => $txt['icon_smiley']),
			array('code' => ';)', 'filename' => 'wink.gif', 'description' => $txt['icon_wink']),
			array('code' => ':D', 'filename' => 'cheesy.gif', 'description' => $txt['icon_cheesy']),
			array('code' => ';D', 'filename' => 'grin.gif', 'description' => $txt['icon_grin']),
			array('code' => '>:(', 'filename' => 'angry.gif', 'description' => $txt['icon_angry']),
			array('code' => ':(', 'filename' => 'sad.gif', 'description' => $txt['icon_sad']),
			array('code' => ':o', 'filename' => 'shocked.gif', 'description' => $txt['icon_shocked']),
			array('code' => '8)', 'filename' => 'cool.gif', 'description' => $txt['icon_cool']),
			array('code' => '???', 'filename' => 'huh.gif', 'description' => $txt['icon_huh']),
			array('code' => '::)', 'filename' => 'rolleyes.gif', 'description' => $txt['icon_rolleyes']),
			array('code' => ':P', 'filename' => 'tongue.gif', 'description' => $txt['icon_tongue']),
			array('code' => ':-[', 'filename' => 'embarrassed.gif', 'description' => $txt['icon_embarrassed']),
			array('code' => ':-X', 'filename' => 'lipsrsealed.gif', 'description' => $txt['icon_lips']),
			array('code' => ':-\\', 'filename' => 'undecided.gif', 'description' => $txt['icon_undecided']),
			array('code' => ':-*', 'filename' => 'kiss.gif', 'description' => $txt['icon_kiss']),
			array('code' => ':\'(', 'filename' => 'cry.gif', 'description' => $txt['icon_cry'])
		);
	else
	{
		if (($temp = cache_get_data('shoutbox_smileys', 3600)) == null)
		{
			$request = $smcFunc['db_query']('', '
				SELECT code, filename, description, smiley_row, hidden
				FROM {db_prefix}smileys
				WHERE hidden IN ({array_int:hidden})
				ORDER BY smiley_row, smiley_order',
				array(
					'hidden' => array(0, 2),
				)
			);
			while ($row = $smcFunc['db_fetch_assoc']($request))
			{
				$row['filename'] = htmlspecialchars($row['filename']);
				$row['description'] = htmlspecialchars($row['description']);
				$row['code'] = htmlspecialchars($row['code']);
				$shoutbox['smileys'][empty($row['hidden']) ? 'normal' : 'popup'][] = $row;
			}
			$smcFunc['db_free_result']($request);
			cache_put_data('shoutbox_smileys', $shoutbox['smileys'], 3600);
		}
		else
			$shoutbox['smileys'] = $temp;
	}

	foreach (array_keys($shoutbox['smileys']) as $location)
	{
		$n = count($shoutbox['smileys'][$location]);
		for ($i = 0; $i < $n; $i++)
		{
			$shoutbox['smileys'][$location][$i]['code'] = addslashes($shoutbox['smileys'][$location][$i]['code']);
			$shoutbox['smileys'][$location][$i]['js_description'] = addslashes($shoutbox['smileys'][$location][$i]['description']);
		}

		if (!empty($shoutbox['smileys'][$location]))
			$shoutbox['smileys'][$location][$n - 1]['last'] = true;
	}

	$shoutbox['bbc'] = array(
		'bold' => array('code' => 'b', 'before' => '[b]', 'after' => '[/b]', 'description' => $txt['bold']),
		'italicize' => array('code' => 'i', 'before' => '[i]', 'after' => '[/i]', 'description' => $txt['italic']),
		'underline' => array('code' => 'u', 'before' => '[u]', 'after' => '[/u]', 'description' => $txt['underline']),
		'strike' => array('code' => 's', 'before' => '[s]', 'after' => '[/s]', 'description' => $txt['strike']),
		'pre' => array('code' => 'pre', 'before' => '[pre]', 'after' => '[/pre]', 'description' => $txt['preformatted']),
		'flash' => array('code' => 'flash', 'before' => '[flash=200,200]', 'after' => '[/flash]', 'description' => $txt['flash']),
		'img' => array('code' => 'img', 'before' => '[img]', 'after' => '[/img]', 'description' => $txt['image']),
		'url' => array('code' => 'url', 'before' => '[url]', 'after' => '[/url]', 'description' => $txt['hyperlink']),
		'email' => array('code' => 'email', 'before' => '[email]', 'after' => '[/email]', 'description' => $txt['insert_email']),
		'ftp' => array('code' => 'ftp', 'before' => '[ftp]', 'after' => '[/ftp]', 'description' => $txt['ftp']),
		'glow' => array('code' => 'glow', 'before' => '[glow=red,2,300]', 'after' => '[/glow]', 'description' => $txt['glow']),
		'shadow' => array('code' => 'shadow', 'before' => '[shadow=red,left]', 'after' => '[/shadow]', 'description' => $txt['shadow']),
		'sup' => array('code' => 'sup', 'before' => '[sup]', 'after' => '[/sup]', 'description' => $txt['superscript']),
		'sub' => array('code' => 'sub', 'before' => '[sub]', 'after' => '[/sub]', 'description' => $txt['subscript']),
		'tele' => array('code' => 'tt', 'before' => '[tt]', 'after' => '[/tt]', 'description' => $txt['teletype']),
		'code' => array('code' => 'code', 'before' => '[code]', 'after' => '[/code]', 'description' => $txt['bbc_code']),
		'quote' => array('code' => 'quote', 'before' => '[quote]', 'after' => '[/quote]', 'description' => $txt['bbc_quote']),
	);

	$shoutbox['allowed_bbc'] = $shoutbox['bbc'];

	return $shoutbox;
}
?>