<?php
/*
 * Simple Portal Extra Shoutbox Options v1.72x
 * Developed for SMF forums c/o Chen Zhen @ http://web-develop.ca
 * Copyright 2013 - 2018 noreply@web-develop.ca
 * This software package is distributed under the terms of the GNU General Public License
 * https://www.gnu.org/licenses/gpl.txt
 */

if (file_exists(dirname(__FILE__) . '/SSI.php') && !defined('SMF'))
	require_once(dirname(__FILE__) . '/SSI.php');
elseif (!defined('SMF'))
	die('<b>Error:</b> Cannot install - please verify you put this in the same place as SMF\'s index.php.');

global $smcFunc, $txt;

list($shoutboxes, $shoutbox_data) = array(array(), array());
$regex = $smcFunc['htmlspecialchars']('(?:^|[\.\!\?]\s+)\w|([^:]\b[i|o]\b)', ENT_QUOTES | ENT_XHTML);
$callback = $smcFunc['htmlspecialchars']('mb_strtoupper($m[0], $encoding)', ENT_QUOTES | ENT_XHTML);
$consec_msg = !empty($txt['sp_shoutbox_consec_msg']) ? $smcFunc['htmlspecialchars']($txt['error_sp_shoutbox_consec_msg']) : '';
$nolinks_msg = !empty($txt['sp_shoutbox_nolinks_msg']) ? $smcFunc['htmlspecialchars']($txt['error_sp_shoutbox_nolinks_msg']) : '';
db_extend('packages');

// create necessary columns if they do not exist
$smcFunc['db_add_column'](
	'{db_prefix}sp_shoutboxes',
	array(
	    'name' => 'groups_read_only',
	    'type' => 'varchar',
	    'size' => 255,
	    'null' => false
	),
	array(),
	'ignore',
	'fatal'
);

$smcFunc['db_add_column'](
	'{db_prefix}sp_shoutboxes',
	array(
	    'name' => 'regex',
	    'type' => 'text',
	    'null' => false
	),
	array(),
	'ignore',
	'fatal'
);

$smcFunc['db_add_column'](
	'{db_prefix}sp_shoutboxes',
	array(
	    'name' => 'regex_callback',
	    'type' => 'text',
	    'null' => false
	),
	array(),
	'ignore',
	'fatal'
);

$smcFunc['db_add_column'](
	'{db_prefix}sp_shoutboxes',
	array(
	    'name' => 'consecutive_msg',
	    'type' => 'text',
	    'null' => false
	),
	array(),
	'ignore',
	'fatal'
);

$smcFunc['db_add_column'](
	'{db_prefix}sp_shoutboxes',
	array(
	    'name' => 'nolinks_msg',
	    'type' => 'text',
	    'null' => false
	),
	array(),
	'ignore',
	'fatal'
);

$smcFunc['db_add_column'](
	'{db_prefix}sp_shoutboxes',
	array(
	    'name' => 'consecutive',
	    'type' => 'int',
	    'size' => 10,
	    'default' => 0,
	    'null' => false
	),
	array(),
	'ignore',
	'fatal'
);

$smcFunc['db_add_column'](
	'{db_prefix}sp_shoutboxes',
	array(
	    'name' => 'charlimit',
	    'type' => 'int',
	    'size' => 10,
	    'default' => 0,
	    'null' => false
	),
	array(),
	'ignore',
	'fatal'
);

$smcFunc['db_add_column'](
	'{db_prefix}sp_shoutboxes',
	array(
	    'name' => 'lowercase',
	    'type' => 'int',
	    'size' => 10,
	    'default' => 0,
	    'null' => false
	),
	array(),
	'ignore',
	'fatal'
);

$smcFunc['db_add_column'](
	'{db_prefix}sp_shoutboxes',
	array(
	    'name' => 'admin_exempt',
	    'type' => 'int',
	    'size' => 10,
	    'default' => 0,
	    'null' => false
	),
	array(),
	'ignore',
	'fatal'
);

// set default values if necessary
$result = $smcFunc['db_query']('','
    SELECT id_shoutbox
    FROM {db_prefix}sp_shoutboxes
    WHERE id_shoutbox > 0',
    array()
);
while ($row = $smcFunc['db_fetch_assoc']($result))
    $shoutboxes[] = $row['id_shoutbox'];

$smcFunc['db_free_result']($result);

foreach ($shoutboxes as $shoutbox_id)
{
    $result = $smcFunc['db_query']('','
	SELECT regex, regex_callback, lowercase, admin_exempt, consecutive, consecutive_msg, nolinks_msg
	FROM {db_prefix}sp_shoutboxes
	WHERE id_shoutbox != {int:current}
	LIMIT 1',
	array(
	    'current' => (int) $shoutbox_id,
	)
    );

    while ($row = $smcFunc['db_fetch_assoc']($result))
	$shoutbox_data[$shoutbox_id] = array(
	    'regex' => !empty($row['regex']) ? $row['regex'] : '',
	    'regex_callback' => !empty($row['regex_callback']) ? $row['regex_callback'] : '',
	    'lowercase' => !empty($row['lowercase']) ? (int)$row['lowercase'] : 0,
	    'admin_exempt' => !empty($row['admin_exempt']) ? (int)$row['admin_exempt'] : 0,
	    'consecutive_msg' => !empty($row['consecutive_msg']) ? $row['consecutive_msg'] : '',
	    'consecutive' => !empty($row['consecutive']) ? (int)$row['consecutive'] : 0,
	    'nolinks_msg' => !empty($row['nolinks_msg']) ? $row['nolinks_msg'] : '',
	);

    $smcFunc['db_free_result']($result);
}

foreach ($shoutbox_data as $id => $shoutbox)
{
    if (empty($shoutbox['regex']))
    {
	$smcFunc['db_query']('','
	    UPDATE {db_prefix}sp_shoutboxes
	    SET regex = {string:regex}
	    WHERE id_shoutbox = {int:id}',
	    array('id' => $id, 'regex' => $regex)
	);
    }

    if (empty($shoutbox['regex_callback']))
    {
	$smcFunc['db_query']('','
	    UPDATE {db_prefix}sp_shoutboxes
	    SET regex_callback = {string:regex_callback}
	    WHERE id_shoutbox = {int:id}',
	    array('id' => $id, 'regex_callback' => $callback)
	);
    }

    if (!isset($shoutbox['lowercase']))
    {
	$smcFunc['db_query']('','
	    UPDATE {db_prefix}sp_shoutboxes
	    SET lowercase = {int:lowercase}
	    WHERE id_shoutbox = {int:id}',
	    array('id' => $id, 'lowercase' => 1)
	);
    }

    if (!isset($shoutbox['admin_exempt']))
    {
	$smcFunc['db_query']('','
	    UPDATE {db_prefix}sp_shoutboxes
	    SET admin_exempt = {int:exempt}
	    WHERE id_shoutbox = {int:id}',
	    array('id' => $id, 'exempt' => 1)
	);
    }

    if (empty($shoutbox['consecutive_msg']))
    {
	$smcFunc['db_query']('','
	    UPDATE {db_prefix}sp_shoutboxes
	    SET consecutive_msg = {string:msg}
	    WHERE id_shoutbox = {int:id}',
	    array('id' => $id, 'msg' => $consec_msg)
	);
    }

    if (empty($shoutbox['nolinks_msg']))
    {
	$smcFunc['db_query']('','
	    UPDATE {db_prefix}sp_shoutboxes
	    SET nolinks_msg = {string:links_msg}
	    WHERE id_shoutbox = {int:id}',
	    array('id' => $id, 'links_msg' => $nolinks_msg)
	);
    }
}
?>